#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h> 
#include <math.h>
#include <hdf.h>

#define DEFAULT_DIM 50
#define DEFAULT_BLKDIMSIZE 8
#define DEFAULT_FNAME "sombrero"


int dimension = DEFAULT_DIM;
int  x_orig, y_orig, z_orig;

float32
	sombrero (x,y,z)
int x,y,z;
{
    double d;
    double dim = (double)dimension;
    int val;
	
    x = x - x_orig;
    y = y - y_orig;
    z = z - z_orig;
    d = ((4.0*M_PI)/dim) * sqrt((double)(x*x + y*y));
    d = (dim/2.0) * cos(d) / (1.0 + d);
    return (float32) ((double)z - d);
}

main(argc, argv)
    int argc;
    char **argv;
{
    extern char *optarg;
    extern int optind;
    int i, j, k, p;
    int x,y,z;
    float32 max,min;
    int dimsizes[3];			/* dimensions of data set */
    int blkdims[3];				/* dimension of array of blocks */
    int blksizes[3];			/* elements along each dim of block */
    int blksize;				/* number of elements in a block */
    int32 blkcoord[3];
    FILE *fp;
    char fname[256];
    float32 *data, *datap;
	
    for (i=0; i<3; i++)
		blksizes[i] = DEFAULT_BLKDIMSIZE;
    strcpy(fname, DEFAULT_FNAME);
	
    while ((p = getopt(argc, argv, "x:y:z:b:d:o:")) != EOF)
		switch (p) {
		case 'x':
			sscanf(optarg, "%d", &blksizes[0]);
			break;
		case 'y':
			sscanf(optarg, "%d", &blksizes[1]);
			break;
		case 'z':
			sscanf(optarg, "%d", &blksizes[2]);
			break;
		case 'b':
			sscanf(optarg, "%d", &blksizes[0]);
			for (i=1; i<3; i++)
				blksizes[i] = blksizes[0];
			break;
		case 'd':
			sscanf(optarg, "%d", &dimension);
			break;
		case 'o':
			strcpy(fname,optarg);
			break; 
		}
    /* initialize veriable */
    x_orig = y_orig = dimension / 2;
    z_orig = dimension / 3;
    min = MAXFLOAT;
    max = -MAXFLOAT;
    
    for (blksize=1, i=0; i<3; i++) {
		dimsizes[i] = dimension;
		blksize *= blksizes[i];
		blkdims[i] = ((dimension - 1) / blksizes[i]) + 1;
    }
	
    /* allocate space for array */
    if ((data = malloc(blksize*sizeof(float32))) == NULL) {
		printf("Error allocating memory\n");
		exit(1);
    }
    
    /* set up to write ndg */
    if (DFSDsetdims(3, dimsizes) == FAIL) {
		HEprint(stderr, 0);
		exit (-1);
    }
    if (DFSDsetblksizes(blksizes) == FAIL) {
		HEprint(stderr, 0);
		exit (-1);
    }
    if (DFSDsetNT(DFNT_FLOAT32 |DFNT_NATIVE) == FAIL) {
		HEprint(stderr, 0);
		exit (-1);
    }
    if (DFSDstartblock(fname) == FAIL) {
		HEprint(stderr, 0);
		exit (-1);
    }
	
    /* loop through blocks, fill them, write them out */
    for (blkcoord[0]=0; blkcoord[0]<blkdims[0]; blkcoord[0]++) {
		for (blkcoord[1]=0; blkcoord[1]<blkdims[1]; blkcoord[1]++) {
			for (blkcoord[2]=0; blkcoord[2]<blkdims[2]; blkcoord[2]++) {
				datap = data;
				for (i=0; i<blksizes[0]; i++) {
					for (j=0; j<blksizes[1]; j++) {
						for (k=0; k<blksizes[2]; k++) {
							x = blkcoord[0]*blksizes[0] + i;
							y = blkcoord[1]*blksizes[1] + j;
							z = blkcoord[2]*blksizes[2] + k;
							if (x>=dimension || y>=dimension || z>=dimension)
								*datap = 0.0;
							else
								*datap = sombrero(x,y,z);
							if (max < *datap)
								max = *datap;
							if (min > *datap)
								min = *datap;
							datap++;
						}
					}
				}
				if (DFSDputblock(blkcoord,&data,1) == FAIL) {
					HEprint(stderr, 0);
					exit (-1);
				}
			}
		}
		printf("Slice starting at x = %d written\n", blkcoord[0]*blksizes[0]);
    }
    if (DFSDsetrange(&max,&min) == FAIL) 
		HEprint(stderr, 0);
	
    if (DFSDendblock() == FAIL) 
		HEprint(stderr, 0);
    printf("Max = %f  Min = %f\n",max,min);
}




